import { FC, useEffect, useState } from 'react';
import { View } from 'react-native';
import { ItemType } from 'react-native-dropdown-picker';
import { BodyMediumText } from '../../../atoms';
import { DeviceItemModel, ThermostatWidgetDetailModel } from '../widgetDetailModels';
import { ModesSelector } from './components/selectors/modesSelector';
import { SetThermostatPoint } from './components/setThermostatPoint';
import { SetThermostatSwitch } from './components/setThermostatSwitch';
import { SpeedSelector } from './components/selectors/speedSelector';
import { styles } from './styles';

const thermostatModes = {
  OFF: '0',
  HOT: '1',
  COLD: '2',
  AUTO: '3',
  FAN_ONLY: '6',
  DRY: '8',
};

const thermostatDeviceType = {
  TEMPERATURE: 'TEMPERATURE',
  THERMOSTAT_FAN_MODE: 'THERMOSTAT_FAN_MODE',
  THERMOSTAT_MODE: 'THERMOSTAT_MODE',
  THERMOSTAT_SETPOINT: 'THERMOSTAT_SETPOINT',
};

const booleanFromMode = {
  [thermostatModes.OFF]: false,
  [thermostatModes.HOT]: true,
  [thermostatModes.COLD]: true,
  [thermostatModes.AUTO]: true,
  [thermostatModes.FAN_ONLY]: true,
  [thermostatModes.DRY]: true,
};

export const ThermostatWidgetDetail: FC<ThermostatWidgetDetailModel> = ({
  id,
  title,
  subtitle,
  setPointTitle,
  setPointSubtitle,
  modesTitle,
  fanModesTitle,
  value,
  onUpdateData,
  childrenItems = [], 
  loadingInfoTextOptions,
  options,
  modesInfoText = {},
  speedsInfoText = {},
  loading = false,
}) => {
  const [state, setState] = useState<string|number|undefined>(value);
  
  const [thermostatMode, setThermostatMode] = useState<DeviceItemModel|undefined>(
    childrenItems.find((item) => item.type === thermostatDeviceType.THERMOSTAT_MODE),
  );
  const [thermostatFanMode, setThermostatFanMode] = useState<DeviceItemModel|undefined>(
    childrenItems.find((item) => item.type === thermostatDeviceType.THERMOSTAT_FAN_MODE),
  );
  const [thermostatAmbientTemperature, setThermostatAmbientTemperature] = useState<DeviceItemModel|undefined>(
    childrenItems.find((item) => item.type === thermostatDeviceType.TEMPERATURE),
  );

  const {
    minTemp,
    maxTemp,
    modes,
    fanModes,
    step,
  } = options;

  const [isEnabled, setIsEnabled] = useState<boolean>(booleanFromMode[thermostatMode?.state || thermostatModes.OFF]);
  const[isThermostatOff, setIsThermostatOff] = useState<boolean>(thermostatMode?.state === thermostatModes.OFF);
  const isOffModeAvailable = modes?.includes(Number(thermostatModes.OFF));
  const modesList = modes?.filter((mode) => mode !== Number(thermostatModes.OFF));
  const fanModesList = fanModes;

  useEffect(() => {
    setThermostatMode(childrenItems.find((item) => item.type === thermostatDeviceType.THERMOSTAT_MODE));
    setThermostatFanMode(childrenItems.find((item) => item.type === thermostatDeviceType.THERMOSTAT_FAN_MODE));
    setThermostatAmbientTemperature(childrenItems.find((item) => item.type === thermostatDeviceType.TEMPERATURE));
  }, [childrenItems]);

  useEffect(() => {
    thermostatMode && setIsEnabled(booleanFromMode[thermostatMode.state]);
    setState(value);
  }, [value]);

  const handleOnChangeThermostatSwitch = (response: boolean) => {
    const value = response ? thermostatModes.HOT : thermostatModes.OFF;
    setIsEnabled(response);
    setIsThermostatOff(!response);
    thermostatMode && onUpdateData?.({
      id: thermostatMode.id,
      value,
    });
  };

  const handleOnChangeThermostatTemperature = (value: number) => {
    onUpdateData?.({ id, value: String(value) });
  };

  const handleOnChangeMode = (item: ItemType<number>) => {
    const value = modesList.find((mode) => mode === item.value);
    (value || value === 0) && thermostatMode && onUpdateData?.({
      id: thermostatMode.id,
      value: value.toString(), 
    });
  };
  const handleOnChangeFanMode = (item: ItemType<number>) => {
    const value = fanModesList.find((mode) => mode === item.value);
    (value || value === 0) && thermostatFanMode && onUpdateData?.({
      id: thermostatFanMode.id,
      value: value.toString(), 
    });
  };

  return (
    <View>
      <View style={styles.setSwitchWrapper}>
        <SetThermostatSwitch
          title={title}
          subtitle={subtitle}
          loadingInfoTextOptions={loadingInfoTextOptions}
          value={!isThermostatOff}
          onChangeValue={handleOnChangeThermostatSwitch}
          loading={loading}
          enabled={isEnabled}
          switchVisible={isOffModeAvailable}
        />
      </View>
      <View style={styles.setPointWrapper}>
        <SetThermostatPoint
          title={setPointTitle}
          subtitle={setPointSubtitle}
          defaultValue={String(state)}
          onChangeValue={handleOnChangeThermostatTemperature}
          step={step}
          minValue={minTemp}
          maxValue={maxTemp}
          ambientTemperature={String(thermostatAmbientTemperature?.state)}
          loading={loading}
          enabled={!isThermostatOff}
        />
      </View>
      {!isThermostatOff &&
          <View style={styles.modesWrapper}>
            {thermostatMode &&
            <View style={styles.modesPicker}>
              <BodyMediumText>{modesTitle}</BodyMediumText>
              <ModesSelector
                modesList={modesList}
                modesTexts={modesInfoText}
                selectedItem={String(thermostatMode.state)}
                onSelectItem={handleOnChangeMode}
                isLoading={loading}
              />
            </View>
            }
            {thermostatFanMode &&
            <View style={styles.modesPicker}>
              <BodyMediumText>{fanModesTitle}</BodyMediumText>
              <SpeedSelector
                speedsList={fanModesList}
                speedsTexts={speedsInfoText}
                selectedItem={String(thermostatFanMode.state)}
                onSelectItem={handleOnChangeFanMode}
                isLoading={loading}
              />
            </View>
            }
          </View>
      }
    </View>
  );
};
